from mwconstants.namespaces import prefixes_to_canonical, standardize_prefix


def standardize_link(link_title):
    """Standardize link format"""
    return link_title.replace(' ', '_').capitalize()


def link_to_namespace(link_title, lang='en', prefix_map=None):
    """Get prefix name (or None if article namespace) for a link."""
    if prefix_map is None:
        prefix_map = prefixes_to_canonical(lang=lang)
    return prefix_map.get(standardize_prefix(link_title))



# See: https://www.mediawiki.org/wiki/User:TJones_(WMF)/Notes/Spaceless_Writing_Systems_and_Wiki-Projects
NON_WHITESPACE_LANGUAGES = (
    'bo', 'dz', 'gan', 'ja', 'km', 'lo', 'my', 'th', 'wuu', 'zh', 'zh-classical', 'zh-yue',  # space-less
    'bug', 'cdo', 'cr', 'hak',  # mixed space/spaceless. could also add: 'jv', 'zh-min-nan'
    'vi')  # spaces but between syllables and not words

WIKIPEDIA_LANGUAGES = (
    'ab', 'ace', 'ady', 'af', 'ak', 'als', 'alt', 'am', 'ami', 'an', 'ang', 'ar', 'arc', 'ary', 'arz', 'as', 'ast',
    'atj', 'av', 'avk', 'awa', 'ay', 'az', 'azb', 'ba', 'ban', 'bar', 'bat-smg', 'bcl', 'be', 'be-x-old', 'bg', 'bh',
    'bi', 'bjn', 'blk', 'bm', 'bn', 'bo', 'bpy', 'br', 'bs', 'bug', 'bxr', 'ca', 'cbk-zam', 'cdo', 'ce', 'ceb', 'ch',
    'chr', 'chy', 'ckb', 'co', 'cr', 'crh', 'cs', 'csb', 'cu', 'cv', 'cy', 'da', 'dag', 'de', 'din', 'diq', 'dsb',
    'dty', 'dv', 'dz', 'ee', 'el', 'eml', 'en', 'eo', 'es', 'et', 'eu', 'ext', 'fa', 'ff', 'fi', 'fiu-vro', 'fj', 'fo',
    'fr', 'frp', 'frr', 'fur', 'fy', 'ga', 'gag', 'gan', 'gcr', 'gd', 'gl', 'glk', 'gn', 'gom', 'gor', 'got', 'gu',
    'guw', 'gv', 'ha', 'hak', 'haw', 'he', 'hi', 'hif', 'hr', 'hsb', 'ht', 'hu', 'hy', 'hyw', 'ia', 'id', 'ie', 'ig',
    'ik', 'ilo', 'inh', 'io', 'is', 'it', 'iu', 'ja', 'jam', 'jbo', 'jv', 'ka', 'kaa', 'kab', 'kbd', 'kbp', 'kcg', 'kg',
    'ki', 'kk', 'kl', 'km', 'kn', 'ko', 'koi', 'krc', 'ks', 'ksh', 'ku', 'kv', 'kw', 'ky', 'la', 'lad', 'lb', 'lbe',
    'lez', 'lfn', 'lg', 'li', 'lij', 'lld', 'lmo', 'ln', 'lo', 'lt', 'ltg', 'lv', 'mad', 'mai', 'map-bms', 'mdf', 'mg',
    'mhr', 'mi', 'min', 'mk', 'ml', 'mn', 'mni', 'mnw', 'mr', 'mrj', 'ms', 'mt', 'mwl', 'my', 'myv', 'mzn', 'na', 'nah',
    'nap', 'nds', 'nds-nl', 'ne', 'new', 'nia', 'nl', 'nn', 'no', 'nov', 'nqo', 'nrm', 'nso', 'nv', 'ny', 'oc', 'olo',
    'om', 'or', 'os', 'pa', 'pag', 'pam', 'pap', 'pcd', 'pcm', 'pdc', 'pfl', 'pi', 'pih', 'pl', 'pms', 'pnb', 'pnt',
    'ps', 'pt', 'pwn', 'qu', 'rm', 'rmy', 'rn', 'ro', 'roa-rup', 'roa-tara', 'ru', 'rue', 'rw', 'sa', 'sah', 'sat',
    'sc', 'scn', 'sco', 'sd', 'se', 'sg', 'sh', 'shi', 'shn', 'si', 'simple', 'sk', 'skr', 'sl', 'sm', 'smn', 'sn',
    'so', 'sq', 'sr', 'srn', 'ss', 'st', 'stq', 'su', 'sv', 'sw', 'szl', 'szy', 'ta', 'tay', 'tcy', 'te', 'tet', 'tg',
    'th', 'ti', 'tk', 'tl', 'tn', 'to', 'tpi', 'tr', 'trv', 'ts', 'tt', 'tum', 'tw', 'ty', 'tyv', 'udm', 'ug', 'uk',
    'ur', 'uz', 've', 'vec', 'vep', 'vi', 'vls', 'vo', 'wa', 'war', 'wo', 'wuu', 'xal', 'xh', 'xmf', 'yi', 'yo', 'za',
    'zea', 'zh', 'zh-classical', 'zh-min-nan', 'zh-yue', 'zu')



from mwconstants.links import link_to_namespace
from mwconstants.constants.c_languages import WIKIPEDIA_LANGUAGES, NON_WHITESPACE_LANGUAGES

__title__ = "mwconstants"
__summary__ = "mwconstants is a package with various data and utilities for processing wikitext."
__url__ = "https://gitlab.wikimedia.org/repos/research/mwconstants"

__version__ = "0.0.1"

__license__ = "MIT License"

__all__ = ["link_to_namespace", "WIKIPEDIA_LANGUAGES", "NON_WHITESPACE_LANGUAGES"]



import copy
import time

import requests

from mwconstants.constants.c_languages import WIKIPEDIA_LANGUAGES
from mwconstants.constants.c_namespaces import DEFAULT_NAMESPACES, NAMESPACE_ALIASES
from mwconstants.languages import get_wiki_sites


def get_all_namespace_prefix_map(lang, project='wikipedia'):
    """Get mapping of namespaces to canonical / possible aliases for a given wiki.

    For return dictionary:
    * Keys are the local names -- i.e. the official names used in parsed article HTML
    * Values include the canonical name -- e.g., User for fr:Utilisateur
    * Values include the list of possible aliases -- e.g., Utilisatrice
    * Together, the key + canonical + aliases make all the valid prefixes for a namespace

    More details: https://www.mediawiki.org/wiki/Help:Namespaces#Localisation

    NOTE: this data can alternatively be extracted from another dump:
    <lang>-<date>-siteinfo-namespaces.json.gz
    """
    session = requests.Session()
    base_url = f"https://{lang}.{project}.org/w/api.php"
    params = {
     "action": "query",
     "meta": "siteinfo",
     "siprop": "namespaces|namespacealiases",
     "format": "json",
     "formatversion": "2"
    }
    result = session.get(url=base_url, params=params)
    result = result.json()

    prefixes = {}
    id_to_name = {}  # e.g., 1 -> Talk; needed for namespace aliases
    if 'namespaces' in result.get('query', {}):
        for ns in result['query']['namespaces'].values():
            try:
                canonical = ns['canonical']
                name = ns['name']
                nid = ns['id']
                id_to_name[nid] = name
                prefixes[name] = {'canonical': canonical}
            except KeyError:  # main namespace has no canonical prefix and we want to skip it
                continue
    if 'namespacealiases' in result.get('query', {}):
        for alias in result['query']['namespacealiases']:
            try:
                nid = alias['id']
                prefix = alias['alias']
                name = id_to_name[nid]
                prefixes[name]['aliases'] = prefixes[name].get('aliases', []) + [prefix]
            except KeyError:
                print(f"Invalid alias: {alias}")

    return prefixes


def gather_all_languages(deduplicate=True):
    """Helper function for gathering namespace data of a given type."""
    wiki_languages = get_wiki_sites('wiki')
    print(f"{len(wiki_languages)} languages: {wiki_languages}")
    namespaces = {}
    for lang in wiki_languages:
        namespaces[lang] = get_all_namespace_prefix_map(lang, 'wikipedia')
        time.sleep(0.25)

    if deduplicate:
        def_ns = copy.deepcopy(namespaces['en'])
        def_canonical_to_name = {ns['canonical']: ns_name for ns_name, ns in def_ns.items()}

        langs_to_remove = []
        for lang in namespaces:
            ns_to_remove = []
            for ns_name in namespaces[lang]:
                ns = namespaces[lang][ns_name]
                # matches English exactly
                if ns == def_ns.get(ns_name):
                    ns_to_remove.append(ns_name)
                else:
                    try:
                        en_equiv = def_canonical_to_name[ns['canonical']]
                    except KeyError:
                        # Special namespaces that don't appear in English -- just remove duplicate canonical if relevant
                        if ns_name == ns['canonical']:
                            ns.pop('canonical')
                        continue
                    # remove official aliases that match any default prefixes (name/canonical/aliases)
                    def_prefixes = def_ns[en_equiv].get('aliases', []) + [en_equiv] + [def_ns[en_equiv]['canonical']]
                    for i in range(len(ns.get('aliases', [])) - 1, -1, -1):
                        if ns['aliases'][i] in def_prefixes:
                            ns['aliases'].pop(i)
                    if 'aliases' in ns and not ns['aliases']:
                        ns.pop('aliases')
                    # only keep canonical if different from name
                    if ns_name == ns['canonical']:
                        ns.pop('canonical')
                    if not ns:
                        ns_to_remove.append(ns_name)
                    elif 'aliases' not in ns and ns['canonical'] in def_prefixes and ns_name in def_prefixes:
                        ns_to_remove.append(ns_name)

            for ns in ns_to_remove:
                namespaces[lang].pop(ns)
            if not namespaces[lang]:
                langs_to_remove.append(lang)

        for lang in langs_to_remove:
            namespaces.pop(lang)

        for ns in def_ns:
            if ns == def_ns[ns]['canonical']:
                def_ns[ns].pop('canonical')

    return namespaces


def standardize_prefix(prefix):
    """Standardize link format"""
    return prefix.split(':', maxsplit=1)[0].replace(' ', '_').capitalize()


def prefixes_to_canonical(lang='en'):
    prefixes = {standardize_prefix(l):'Interlanguage' for l in WIKIPEDIA_LANGUAGES}
    for ns in DEFAULT_NAMESPACES:
        canonical = DEFAULT_NAMESPACES[ns].get('canonical', ns)
        prefixes[standardize_prefix(ns)] = canonical
        prefixes[standardize_prefix(canonical)] = canonical
        for alias in DEFAULT_NAMESPACES[ns].get('aliases', []):
            prefixes[standardize_prefix(alias)] = canonical

    for ns in NAMESPACE_ALIASES.get(lang, {}):
        canonical = NAMESPACE_ALIASES[lang][ns].get('canonical', ns)
        prefixes[standardize_prefix(ns)] = canonical
        prefixes[standardize_prefix(canonical)] = canonical
        for alias in NAMESPACE_ALIASES[lang][ns].get('aliases', []):
            prefixes[standardize_prefix(alias)] = canonical

    return prefixes


if __name__ == '__main__':
    print(gather_all_languages())  # Wikipedia namespaces



import time

import requests

from mwconstants.languages import get_wiki_sites


def get_valid_img_options(lang):
    """Get official list of acceptable image formatting tags -- e.g., frameless, top, etc."""
    session = requests.Session()
    base_url = f"https://{lang}.wikipedia.org/w/api.php"
    params = {
        "action": "query",
        "meta": "siteinfo",
        "siprop": "magicwords",
        "format": "json",
        "formatversion": "2",
    }
    result = session.get(url=base_url, params=params)
    result = result.json()

    img_keywords = []
    img_param_names = []
    img_startswith = []
    img_endswith = []
    if "magicwords" in result.get("query", {}):
        for magic in result["query"]["magicwords"]:
            if magic['name'].startswith('img_'):
                if not magic['case-sensitive']:
                    # we don't handle case insensitive so...
                    print(f"warning case-insensitive for {lang}: {magic}")
                for tag in magic['aliases']:
                    if '=' in tag:
                        img_param_names.append((magic['name'], tag.split('=')[0]))
                    elif tag.endswith('$1'):
                        img_startswith.append((magic['name'], tag[:-2]))
                    elif tag.startswith('$1'):
                        img_endswith.append((magic['name'], tag[2:]))
                    elif '$' in tag:
                        print("Didn't expect this:", tag)
                    else:
                        img_keywords.append((magic['name'], tag))
    return {'keywords': set(img_keywords),
            'params': set(img_param_names),
            'startswith': set(img_startswith),
            'endswith': set(img_endswith)}


def get_img_options(project='wiki'):
    """
    Utility for generating language-specific lists of valid media formatting options on a Wikimedia project.

    For example, to know that [[File:image.png|thumb]] has `thumb` formatting as opposed to a caption that reads `thumb`
    we must know that `thumb` is a valid image formatting tag and appears as a stand-alone keyword. Any parameters that
    do not match a valid image formatting tag pattern are assumed to be the image caption.

    See: https://www.mediawiki.org/wiki/Help:Images#Syntax
    """

    wiki_languages = get_wiki_sites(project)
    print(f"{len(wiki_languages)} languages: {wiki_languages}")
    img_options = {}
    for lang in wiki_languages:
        img_options[lang] = get_valid_img_options(lang)
        time.sleep(0.25)

    # set -> list: lists are short enough that lookups still quite fast
    for lang in img_options:
        for param_type in img_options[lang]:
            img_options[lang][param_type] = sorted(img_options[lang][param_type])

    # separate out english tags, which are valid for any language
    # this greatly reduces the size of the final list
    default_option_tags = img_options['en']

    to_remove = []
    for lang in img_options:
        for param_type in default_option_tags:
            for tag in default_option_tags[param_type]:
                img_options[lang][param_type].remove(tag)
            if not img_options[lang][param_type]:
                img_options[lang].pop(param_type)
        if not img_options[lang]:
            to_remove.append(lang)

    for lang in to_remove:
        img_options.pop(lang)

    return default_option_tags, img_options


if __name__ == "__main__":
    print(get_img_options('wiki'))  # Wikipedia image formatting options
